"""
Contains the test cases for openedx_caliper_tracking
application logs delivery to Rest API.
"""
import json

import mock
from django.conf import settings
from django.test import TestCase, override_settings

from openedx_caliper_tracking.processor import CaliperProcessor
from openedx_caliper_tracking.tests import TEST_DIR_PATH


class CaliperDeliveryTestCase(TestCase):
    """
    Test that determines that caliper app is properly delivering the data
    to an external API
    """

    def setUp(self):
        input_file = '{}/current/{}'.format(
            TEST_DIR_PATH,
            'book.json'
        )
        with open(input_file) as current:
            self.event = json.loads(current.read())

    @mock.patch(
        'openedx_caliper_tracking.processor.deliver_caliper_event'
    )
    def test_event_is_not_called_without_settings(self, delivery_mock):
        """
        Test that API call shouldn't fire if the following settings

            ENABLE_CALIPER_EVENTS_DELIVERY,
            CALIPER_DELIVERY_ENDPOINT,
            CALIPER_DELIVERY_AUTH_TOKEN

        are not given.
        """
        if settings.FEATURES.get('ENABLE_CALIPER_EVENTS_DELIVERY'):
            settings.FEATURES.pop('ENABLE_CALIPER_EVENTS_DELIVERY')

        if hasattr(settings, 'CALIPER_DELIVERY_ENDPOINT'):
            del settings.CALIPER_DELIVERY_ENDPOINT

        if hasattr(settings, 'CALIPER_DELIVERY_AUTH_TOKEN'):
            del settings.CALIPER_DELIVERY_AUTH_TOKEN

        CaliperProcessor().__call__(self.event)
        self.assertFalse(delivery_mock.called)

    @mock.patch(
        'openedx_caliper_tracking.processor.deliver_caliper_event'
    )
    @override_settings(
        CALIPER_DELIVERY_ENDPOINT='http://localhost:3000',
        CALIPER_DELIVERY_AUTH_TOKEN='test_auth_token',
        FEATURES={'ENABLE_CALIPER_EVENTS_DELIVERY': False}
    )
    def test_event_is_not_called_with_unset_delivery_flag(self, delivery_mock):
        """
        Test that API call shouldn't fire if the feature flag ENABLE_CALIPER_EVENTS_DELIVERY is not set.
        """
        CaliperProcessor().__call__(self.event)
        self.assertFalse(delivery_mock.called)

    @mock.patch(
        'openedx_caliper_tracking.processor.deliver_caliper_event',
        autospec=True
    )
    @override_settings(
        CALIPER_DELIVERY_ENDPOINT='http://localhost:3000',
        CALIPER_DELIVERY_AUTH_TOKEN='test_auth_token',
        FEATURES={'ENABLE_CALIPER_EVENTS_DELIVERY': True}
    )
    def test_event_is_called_with_settings(self, delivery_mock):
        """
        Test that  caliper event is delivered with all required settings given.
        """
        CaliperProcessor().__call__(self.event)
        self.assertTrue(delivery_mock.called)



"""
Contains the test cases for
openedx_caliper_tracking application transformers.
"""
import json
import os

import mock
from django.test import TestCase

from openedx_caliper_tracking.base_transformer import base_transformer
from openedx_caliper_tracking.caliper_config import EVENT_MAPPING
from openedx_caliper_tracking.tests import TEST_DIR_PATH


class CaliperTransformationTestCase(TestCase):
    """
    Data driven test case that tests all transformers for expected output
    The test case compares the files in the current/ directory
    and compares them with their corresponding files in the expected/ directory
    """

    maxDiff = None

    @mock.patch(
        'openedx_caliper_tracking.utils.get_username_from_user_id',
        return_value='honor',
        autospec=True
    )
    @mock.patch(
        'openedx_caliper_tracking.utils.get_team_url_from_team_id',
        return_value=(
            "http://localhost:18000/courses/course-v1:edX+DemoX+Demo_Course/teams/"
            "#teams/Topic1ID/check-3429fc5983a84c8c8366a4477b03d91c"
        ),
        autospec=True
    )
    def test_caliper_transformers(self, *args):
        """
        Tests whether all the caliper transformers are working as expected
        """
        test_files = [test for test in os.listdir(
            '{}/current/'.format(TEST_DIR_PATH)) if test.endswith(".json")]

        for current_file in test_files:
            input_file = '{}/current/{}'.format(
                TEST_DIR_PATH,
                current_file
            )
            output_file = '{}/expected/{}'.format(
                TEST_DIR_PATH,
                current_file
            )
            with self.settings(LMS_ROOT_URL='http://localhost:18000'):
                with open(input_file) as current, open(output_file) as expected:
                    event = json.loads(current.read())
                    expected_event = json.loads(expected.read())

                    expected_event.pop('id')

                    caliper_event = base_transformer(event)
                    related_function = EVENT_MAPPING[event.get('event_type')]
                    caliper_event = related_function(event, caliper_event)

                    caliper_event.pop('id')

                    self.assertDictEqual(caliper_event, expected_event)



"""
Utils required in transformers
"""
import json
import logging
from smtplib import SMTPException

from dateutil.parser import parse
from django.contrib.auth import get_user_model
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse
from django.urls.exceptions import NoReverseMatch
from django.template.loader import get_template

log = logging.getLogger(__name__)
UTC_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%f'


def convert_datetime(current_datetime):
    """
    Convert provided datetime into UTC format
    @param datetime: datetime string.
    :return: UTC formatted datetime string.
    """

    # convert current_datetime to a datetime object if it is string
    if type(current_datetime) in (str, unicode):
        current_datetime = parse(current_datetime)

    utc_offset = current_datetime.utcoffset()
    utc_datetime = current_datetime - utc_offset

    formatted_datetime = '{}{}'.format(
        utc_datetime.strftime(UTC_DATETIME_FORMAT)[:-3], 'Z'
    )
    return formatted_datetime


def get_username_from_user_id(user_id):
    """
    @param : user_id
    :return: username from the given user_id.
    """

    User = get_user_model()
    user = User.objects.get(id=user_id)
    return str(user.username)


def get_user_link_from_username(username):
    try:
        link = '{lms_url}{profile_link}'.format(
            lms_url=settings.LMS_ROOT_URL,
            profile_link=str(reverse(
                'learner_profile',
                kwargs={'username': username}
            ))
        )
    except NoReverseMatch:
        link = '{lms_url}/u/{username}'.format(
            lms_url=settings.LMS_ROOT_URL,
            username=username
        )
    return link


def get_topic_id_from_team_id(team_id):
    """
    :param team_id: extracting from event logs
    :return: topic_id for making team url
    """
    from lms.djangoapps.teams.models import CourseTeam
    user_team = CourseTeam.objects.get(team_id=team_id)

    return user_team.topic_id


def get_team_url_from_team_id(referer, team_id):
    """
    :param referer: extract from event logs
    :param team_id: extract from event logs
    :return: team url
    """
    topic_id = get_topic_id_from_team_id(team_id)
    object_link = '{referrer}#teams/{topic_id}/{team_id}'.format(
        referrer=referer,
        topic_id=topic_id,
        team_id=team_id
    )
    return object_link


def get_certificate_url(user_id, course_id):
    """
    This function takes user_id and course_id as parameters
    makes a reverse url for certificate generated by that user.

    :return: A URI of required certificate.
    """
    certificate_uri = '{lms_url}{certificate_link}'.format(
        lms_url=settings.LMS_ROOT_URL,
        certificate_link=str(reverse(
            'certificates:html_view',
            kwargs={
                'user_id': user_id,
                'course_id': course_id
            }
        ))
    )
    return certificate_uri


def send_notification(key, data, subject, from_email, dest_emails):
    """
    Send an email.

    params:
        key - Email template will be selected on the basis of key
        data - Dict containing context/data for the template
        subject - Email subject
        from_email - Email address to send email
        dest_emails - List of destination emails

    return: a boolean variable indicating email response.
    """
    TEMPLATE_PATH = '{key}_email.html'
    content = json.dumps(data)
    email_template_path = TEMPLATE_PATH.format(key=key)
    html_content = get_template(email_template_path).render(data)
    msg = EmailMultiAlternatives(subject, content, from_email, dest_emails)
    msg.attach_alternative(html_content, "text/html")
    try:
        response = msg.send()
        log.info(
            'Email has been sent from "%s" to "%s" for content %s.',
            from_email,
            dest_emails,
            content
        )
        return response
    except SMTPException:
        log.exception(
            'Unable to send an email from "%s" to %s for content "%s".',
            from_email,
            dest_emails,
            content
        )
        return False
